@@ -21,6 +21,7 @@
 #include <libxml/xmlerror.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xpathInternals.h>
+#include <libxml/xpath.h>
 #include "xslt.h"
 #include "xsltInternals.h"
 #include "xsltutils.h"
@@ -37,9 +38,9 @@ xsltInitDocKeyTable(xsltTransformContextPtr ctxt, const xmlChar *name,
                     const xmlChar *nameURI);
 
 /************************************************************************
- * 									*
- * 			Type functions 					*
- * 									*
+ *									*
+ *			Type functions					*
+ *									*
  ************************************************************************/
 
 /**
@@ -158,7 +159,7 @@ xsltFreeKeyTable(xsltKeyTablePtr keyt) {
     if (keyt->nameURI != NULL)
 	xmlFree(keyt->nameURI);
     if (keyt->keys != NULL)
-	xmlHashFree(keyt->keys, 
+	xmlHashFree(keyt->keys,
 		    (xmlHashDeallocator) xmlXPathFreeNodeSet);
     memset(keyt, -1, sizeof(xsltKeyTable));
     xmlFree(keyt);
@@ -182,9 +183,9 @@ xsltFreeKeyTableList(xsltKeyTablePtr keyt) {
 }
 
 /************************************************************************
- * 									*
- * 		The interpreter for the precompiled patterns		*
- * 									*
+ *									*
+ *		The interpreter for the precompiled patterns		*
+ *									*
  ************************************************************************/
 
 
@@ -356,14 +357,22 @@ xsltAddKey(xsltStylesheetPtr style, const xmlChar *name,
     *   Maybe a search for "$", if it occurs outside of quotation
     *   marks, could be sufficient.
     */
+#ifdef XML_XPATH_NOVAR
+    key->comp = xsltXPathCompileFlags(style, pattern, XML_XPATH_NOVAR);
+#else
     key->comp = xsltXPathCompile(style, pattern);
+#endif
     if (key->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 		"xsl:key : XPath pattern compilation failed '%s'\n",
 		         pattern);
 	if (style != NULL) style->errors++;
     }
+#ifdef XML_XPATH_NOVAR
+    key->usecomp = xsltXPathCompileFlags(style, use, XML_XPATH_NOVAR);
+#else
     key->usecomp = xsltXPathCompile(style, use);
+#endif
     if (key->usecomp == NULL) {
 	xsltTransformError(NULL, style, inst,
 		"xsl:key : XPath pattern compilation failed '%s'\n",
@@ -474,7 +483,7 @@ xsltInitDocKeyTable(xsltTransformContextPtr ctxt, const xmlChar *name,
                     const xmlChar *nameURI)
 {
     xsltStylesheetPtr style;
-    xsltKeyDefPtr keyd;
+    xsltKeyDefPtr keyd = NULL;
     int found = 0;
 
 #ifdef KEY_INIT_DEBUG
@@ -504,7 +513,7 @@ fprintf(stderr, "xsltInitDocKeyTable %s\n", name);
 	XSLT_TRACE(ctxt,XSLT_TRACE_KEYS,xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltInitDocKeyTable: did not found %s\n", name));
 #endif
-	xsltTransformError(ctxt, NULL, keyd->inst,
+	xsltTransformError(ctxt, NULL, keyd? keyd->inst : NULL,
 	    "Failed to find key definition for %s\n", name);
 	ctxt->state = XSLT_STATE_STOPPED;
         return(-1);
@@ -517,10 +526,13 @@ fprintf(stderr, "xsltInitDocKeyTable %s done\n", name);
 
 /**
  * xsltInitAllDocKeys:
+ * @ctxt: transformation context
  *
  * INTERNAL ROUTINE ONLY
  *
  * Check if any keys on the current document need to be computed
+ *
+ * Returns 0 in case of success, -1 in case of failure
  */
 int
 xsltInitAllDocKeys(xsltTransformContextPtr ctxt)
